<?
class CWD_YandexKassa {
	
	const SERVER_DEMO = 'https://penelope-demo.yamoney.ru:8083/webservice/mws/api/#ACTION#';
	const SERVER_WORK = 'https://penelope.yamoney.ru/webservice/mws/api/#ACTION#';
	
	private $DemoMode = true;
	
	private $ShopID = false;
	private $Certificate = false;
	private $PrivateKey = false;
	private $Password = false;
	
	/**
	 *	Создание объекта
	 */
	public function __construct($ShopID, $Certificate, $PrivateKey, $Password){
		if(!empty($ShopID) && IntVal($ShopID)>0) {
			$this->ShopID = $ShopID;
		} else {
			throw new Exception('ShopID is empty.');
		}
		if(!empty($Certificate) && is_file($Certificate) && is_readable ($Certificate)>0 && filesize($Certificate)>0) {
			$this->Certificate = $Certificate;
		} else {
			throw new Exception('Certificate file is missing or empty.');
		}
		if(!empty($PrivateKey) && is_file($PrivateKey) && is_readable ($PrivateKey)>0 && filesize($PrivateKey)>0) {
			$this->PrivateKey = $PrivateKey;
		} else {
			throw new Exception('Private.key is missing or empty.');
		}
		if(!empty($Password)) {
			$this->Password = $Password;
		} else {
			throw new Exception('Password is empty.');
		}
	}
	
	/**
	 *	Отладочный вывод
	 */
	public function X($Value) {
		print '<pre>';
		print (is_array($Value) ? print_r($Value,1) : htmlspecialchars($Value));
		print '</pre>';
	}
	
	/**
	 *	Удаление из XML строки <?xml version="1.0" encoding="UTF-8" ?>
	 */
	private function RemoveXmlHeader($Value) {
		return preg_replace('#^<\?xml version="[0-9\.]+" encoding="UTF-8" \?>#i','',$Value);
	}
	
	/**
	 *	SimpleXML => Array
	 */
	private function XmlToArray($Xml,$Array=array()) {
		foreach ((array)$Xml as $Index => $Value) {
			$Array[$Index] = is_object($Value) ? $this->XmlToArray($Value) : $Value;
		}
		return $Array;
	}
	
	/**
	 *	Функция выполнения запроса к серверу Яндекс
	 */
	private function Request($Action, $Content){
		$Result = false;
		$Curl = curl_init();
		$arParams = array(
			CURLOPT_URL => str_replace('#ACTION#',$Action,($this->DemoMode===false?self::SERVER_WORK:self::SERVER_DEMO)),
			CURLOPT_RETURNTRANSFER => true,
			CURLOPT_TIMEOUT => 30,
			CURLOPT_POST => true,
			CURLOPT_CRLF => true,
			CURLOPT_HTTPHEADER => array('Content-type: application/x-www-form-urlencoded'),
			CURLOPT_POSTFIELDS => http_build_query($Content),
			CURLOPT_SSL_VERIFYPEER => false,
			CURLOPT_SSL_VERIFYHOST => false,
			CURLOPT_SSLCERT => $this->Certificate,
			CURLOPT_SSLKEY => $this->PrivateKey,
			CURLOPT_SSLCERTPASSWD => $this->Password,
		);
		curl_setopt_array($Curl, $arParams);
		try {
			$Result = curl_exec($Curl);
			if (!$Result) {
				var_dump(curl_error($Curl));
			} else {
				$Result = $this->XmlToArray(simplexml_load_string($Result));
			}
			curl_close($Curl);
		} catch (Exception $E) {
			echo $E;
		}
		return $Result;
	}
	
	/**
	 *	Получение даты и времени в формате Яндекс
	 */
	private function DateTime($Difference=0){
		$Difference = IntVal($Difference);
		if($Difference<=0) {
			$Difference = 0;
		}
		return date('Y-m-d\TH:i:s.000\Z', time()-$Difference);
	}
	
	/**
	 *	Получение списка заказов
	 */
	public function ListOrders($OrderID=false){
		$DateTime = $this->DateTime();
		$arParams = array(
			'requestDT' => $DateTime,
			'outputFormat' => 'XML',
			'shopId' => $this->ShopID,
			'orderCreatedDatetimeLessOrEqual' => $DateTime,
			'orderCreatedDatetimeGreaterOrEqual' => $this->DateTime(30*24*60*60),
		);
		if($OrderID>0) {
			$arParams['orderNumber'] = $OrderID;
		} else {
			$arParams['orderCreatedDatetimeLessOrEqual'] = $DateTime;
			$arParams['orderCreatedDatetimeGreaterOrEqual'] = $this->DateTime(30*24*60*60);
		}
		$Result = $this->Request('listOrders',$arParams);
		return $Result;
	}
	
	/**
	 *	Получение списка возвратов
	 */
	public function ListReturns(){
		$DateTime = $this->DateTime();
		$arParams = array(
			'requestDT' => $DateTime,
			'outputFormat' => 'XML',
			'shopId' => $this->ShopID,
			'orderCreatedDatetimeLessOrEqual' => $DateTime,
			'orderCreatedDatetimeGreaterOrEqual' => $this->DateTime(30*24*60*60),
			'from' => $this->DateTime(30*24*60*60),
			'till' => $DateTime,
		);
		$Result = $this->Request('listReturns',$arParams);
		return $Result;
	}
	
	/**
	 *	Подтверждение платежа
	 */
	public function ConfirmPayment($OrderID, $OrderPrice){
		$DateTime = $this->DateTime();
		$arParams = array(
			'requestDT' => $DateTime,
			'orderId' => $OrderID,
			'amount' => $OrderPrice,
			'currency' => 'RUB',
		);
		$Result = $this->Request('confirmPayment',$arParams);
		if(is_array($Result) && $Result['@attributes']['status']=='0' && $Result['@attributes']['error']=='0') {
			return true;
		}
		return false;
	}
	
	/**
	 *	Отмена платежа
	 */
	public function CancelPayment($OrderID){
		$DateTime = $this->DateTime();
		$arParams = array(
			'requestDT' => $DateTime,
			'orderId' => $OrderID,
		);
		$Result = $this->Request('cancelPayment',$arParams);
		if(is_array($Result) && $Result['@attributes']['status']=='0' && $Result['@attributes']['error']=='0') {
			return true;
		}
		return false;
	}
	
	/**
	 *	Получение ID транзации по ID заказа
	 */
	public function GetTransactionID($OrderID){
		$Result = $this->ListOrders($OrderID);
		if(is_array($Result['order']['@attributes']) && $Result['order']['@attributes']['orderNumber']==$OrderID && !empty($Result['order']['@attributes']['invoiceId'])) {
			return $Result['order']['@attributes']['invoiceId'];
		}
		return false;
	}
	
}
?>